package com.cultural.heritage.service.wx.impl;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.config.WxOpenConfig;
import com.cultural.heritage.config.WxPayConfig;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.order.BookingOrderQueryRequest;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.model.enums.GoodTypeEnum;
import com.cultural.heritage.service.good.AppointmentDateService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.order.*;
import com.cultural.heritage.service.wx.WeChatService;
import com.cultural.heritage.utils.RefundUtils;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.jsapi.model.Amount;
import com.wechat.pay.java.service.payments.jsapi.model.Payer;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayRequest;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 玄德
 */
@Slf4j
@Service
public class WeChatServiceImpl implements WeChatService {

    @Resource
    private WxPayConfig wxPayConfig;

    @Resource
    private WxOpenConfig wxOpenConfig;

    @Resource
    private OrderService orderService;

    @Resource
    private GoodService goodService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private RefundRecordService refundRecordService;

    @Resource
    private AppointmentDateService appointmentDateService;

    @Resource
    private PendingServiceGoodService pendingServiceGoodService;

    @Resource
    private PendingServiceOrderService pendingServiceOrderService;


    /**
     * 请求参数
     */
    public static RequestParam requestParam = null;


    /**
     * 微信支付
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized PrepayWithRequestPaymentResponse createPayment(String orderId, String miniOpenId, BigDecimal amount) {
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        PrepayRequest request = new PrepayRequest();
        // 金额
        Amount WxAmount = new Amount();
        WxAmount.setTotal(amount.movePointRight(2).intValue());
        WxAmount.setCurrency("CNY");
        request.setAmount(WxAmount);
        // 公众号id
        request.setAppid(wxPayConfig.getAppId());
        // 商户号
        request.setMchid(wxPayConfig.getMerchantId());
        // 支付者信息
        Payer payer = new Payer();
        payer.setOpenid(miniOpenId);
        request.setPayer(payer);
        // 获取订单号
        Order order = orderService.getById(orderId);
        String orderNumber = order.getOrderNumber();
        // 描述
        request.setDescription("订单号：" + orderNumber);
        // 微信回调地址
        request.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/payment/callback");
        // 商户订单号
        request.setOutTradeNo(orderNumber);
        //返回数据，前端调起支付
        return wxPayConfig.getJsapiServiceExtension().prepayWithRequestPayment(request);
    }



    /**
     * 获取支付回调信息
     */
    @Override
    public Transaction getTransactionInfo(HttpServletRequest request) {
        NotificationParser notificationParser = getNotificationParser(request);
        return notificationParser.parse(requestParam, Transaction.class);
    }


    /**
     * 支付回调
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean paymentCallback(Transaction transaction) {
        System.out.println("---------------------------微信支付回调（开始）-------------------------------");
        // 获取订单信息
        String orderIdByString = transaction.getOutTradeNo();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderIdByString);
        Order order = orderService.getOne(queryWrapper);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + transaction.getOutTradeNo());
        }
        // 修改订单状态
        order.setOrderStatus(OrderStatusConstant.PENDING_SHIPMENT);
        order.setUpdateTime(DateUtil.date());
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态修改失败");

        // 生成服务类订单待处理记录
        GoodTypeEnum goodTypeEnum = GoodTypeEnum.getEnumByValue(order.getOrderType());
        if (GoodTypeEnum.SERVICE.equals(goodTypeEnum)) {

            // 封装map集合（键：服务类商品待处理id, 值：服务类商品待处理记录）
            Map<String, PendingServiceGood> pendingGoodMap = new HashMap<>();
            List<PendingServiceGood> pendingServiceGoodList = pendingServiceGoodService.list();
            for (PendingServiceGood pendingServiceGood : pendingServiceGoodList) {
                BookingOrderQueryRequest bookingOrderQueryRequest = new BookingOrderQueryRequest();
                BeanUtils.copyProperties(pendingServiceGood, bookingOrderQueryRequest);
                pendingGoodMap.put(bookingOrderQueryRequest.toString(), pendingServiceGood);
            }

            QueryWrapper<OrderItems> orderItemsQueryWrapper = new QueryWrapper<>();
            orderItemsQueryWrapper.eq("orderId", order.getId());
            List<OrderItems> orderItemsList = orderItemService.list(orderItemsQueryWrapper);

            List<PendingServiceOrder> pendingServiceOrderList = new ArrayList<>();
            for (OrderItems orderItems : orderItemsList) {
                PendingServiceOrder pendingServiceOrder = new PendingServiceOrder();
                pendingServiceOrder.setOrderItemId(orderItems.getId());
                pendingServiceOrder.setOrderNumber(order.getOrderNumber());
                pendingServiceOrder.setUserId(order.getUserId());
                pendingServiceOrder.setName(order.getContactsSnapshot().getName());
                pendingServiceOrder.setPhone(order.getContactsSnapshot().getPhone());
                pendingServiceOrder.setQuantity(orderItems.getQuantity());
                pendingServiceOrder.setItemTotalAmount(orderItems.getItemTotalAmount());
                pendingServiceOrder.setOrderItemStatus(order.getOrderStatus());
                BookingOrderQueryRequest bookingOrderQueryRequest = new BookingOrderQueryRequest(orderItems.getGoodSnapshot().getId(), orderItems.getReservationDate(), orderItems.getTimeSlot());
                PendingServiceGood pendingServiceGood = pendingGoodMap.get(bookingOrderQueryRequest.toString());
                pendingServiceOrder.setPendingId(pendingServiceGood.getId());

                pendingServiceOrderList.add(pendingServiceOrder);
            }

            boolean saveBatch = pendingServiceOrderService.saveBatch(pendingServiceOrderList);
            ThrowUtils.throwIf(!saveBatch, ErrorCode.OPERATION_ERROR, "服务类订单待处理记录批量生成失败");

        }

        System.out.println("---------------------------微信支付回调（结束）-------------------------------");
        return true;
    }


    /**
     * 全额退款
     */
    @Override
    public Refund refundPayment(String orderId, BigDecimal amount) {
        // 获取订单号
        Order order = orderService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "该订单不存在");
        String orderNumber = order.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        String outRefundNo = RefundUtils.generateRefundNo();
        createRequest.setOutRefundNo(outRefundNo);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();
        long refundAmount = amount.movePointRight(2).intValue();

        amountReq.setRefund(refundAmount);
        amountReq.setTotal(refundAmount);
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);
        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);
        return refund;
    }


    /**
     * 部分退款
     */
    @Override
    public Refund refundPartPayment(String orderItemsId, BigDecimal amount) {
        // 获取订单明细
        OrderItems orderItems = orderItemService.getById(orderItemsId);
        ThrowUtils.throwIf(orderItemsId == null, ErrorCode.OPERATION_ERROR, "订单明细不存在");
        // 获取订单明细所属的订单
        Order order = orderService.getById(orderItems.getOrderId());
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        // 判断该订单明细是否已经退款
        QueryWrapper<PendingServiceOrder> pendingServiceOrderQueryWrapper = new QueryWrapper<>();
        pendingServiceOrderQueryWrapper.eq("orderItemId", orderItemsId);
        PendingServiceOrder pendingServiceOrder = pendingServiceOrderService.getOne(pendingServiceOrderQueryWrapper);
        ThrowUtils.throwIf(pendingServiceOrder == null || !pendingServiceOrder.getOrderItemStatus().equals(OrderStatusConstant.PENDING_SHIPMENT), ErrorCode.OPERATION_ERROR, "服务类订单待处理记录不存在或者当前订单明细状态错误");

        String orderNumber = order.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        String outRefundNo = RefundUtils.generateRefundPartNo(orderItemsId);
        createRequest.setOutRefundNo(outRefundNo);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/part/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();



        BigDecimal refundAmount;
        CouponSnapshot couponSnapshot = order.getCouponSnapshot();
        if (couponSnapshot == null) {
            refundAmount = amount;
        } else {
            // 判断是否为最后一个商品，如果是，则将剩余金额全部退款
            QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("orderId", order.getId());
            long itemCount = orderItemService.count(queryWrapper);

            QueryWrapper<RefundRecord> recordQueryWrapper = new QueryWrapper<>();
            recordQueryWrapper.eq("outTradeNo", order.getOrderNumber());
            long refundCount = refundRecordService.count(recordQueryWrapper);

            if (itemCount - refundCount == 1) {
                List<RefundRecord> refundRecords = refundRecordService.list(recordQueryWrapper);
                BigDecimal refundTotalAmount = new BigDecimal("0.00");
                for (RefundRecord refundRecord : refundRecords) {
                    BigDecimal recordRefundAmount = refundRecord.getRefundAmount();
                    refundTotalAmount = refundTotalAmount.add(recordRefundAmount);
                }
                refundAmount = order.getTotalAmount().subtract(refundTotalAmount);
            } else {
                BigDecimal couponAmount = couponSnapshot.getConditionAmount();
                BigDecimal totalAmount = order.getTotalAmount().add(couponAmount);
                BigDecimal itemShare = amount.divide(totalAmount, 10, RoundingMode.HALF_UP);
                BigDecimal itemCouponRefund = itemShare.multiply(couponAmount);
                refundAmount = amount.subtract(itemCouponRefund);
            }
        }

        refundAmount = refundAmount.movePointRight(2).setScale(0, RoundingMode.HALF_UP);
        amountReq.setRefund(refundAmount.longValue());
        amountReq.setTotal(order.getTotalAmount().movePointRight(2).longValue());
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);
        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);

        return refund;
    }


    /**
     * 获取退款回调信息
     */
    @Override
    public RefundNotification getRefundInfo(HttpServletRequest request) {
        NotificationParser notificationParser = getNotificationParser(request);
        return notificationParser.parse(requestParam, RefundNotification.class);
    }


    /**
     * 全额退款回调
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean refundCallback(RefundNotification refundNotification) {
        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderIdByString = refundNotification.getOutTradeNo();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderIdByString);
        Order order = orderService.getOne(queryWrapper);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + orderIdByString);
        }

        // 修改订单状态
        order.setOrderStatus(OrderStatusConstant.PAYMENT_REFUNDED);
        order.setUpdateTime(DateUtil.date());
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态修改失败");

        // 生成退款记录
        RefundRecord refundRecord = new RefundRecord();
        refundRecord.setOutTradeNo(refundNotification.getOutTradeNo());
        refundRecord.setOutRefundNo(refundNotification.getOutRefundNo());
        refundRecord.setRefundAmount(order.getTotalAmount());

        boolean save = refundRecordService.save(refundRecord);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "退款记录生成失败");


        GoodTypeEnum orderTypeEnum = GoodTypeEnum.getEnumByValue(order.getOrderType());
        if (GoodTypeEnum.SERVICE.equals(orderTypeEnum)) {
            // 更新服务类订单待处理记录
            QueryWrapper<PendingServiceOrder> orderQueryWrapper = new QueryWrapper<>();
            orderQueryWrapper.eq("orderNumber", order.getOrderNumber());
            List<PendingServiceOrder> pendingServiceOrderList = pendingServiceOrderService.list(orderQueryWrapper);
            for (PendingServiceOrder pendingServiceOrder : pendingServiceOrderList) {
                pendingServiceOrder.setOrderItemStatus(OrderStatusConstant.PAYMENT_REFUNDED);
            }
            boolean result = pendingServiceOrderService.updateBatchById(pendingServiceOrderList);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类订单待处理记录批量更新失败");
        } else {

            // 恢复商品库存
            QueryWrapper<OrderItems> orderItemsQueryWrapper = new QueryWrapper<>();
            orderItemsQueryWrapper.eq("orderId", order.getId());
            List<OrderItems> orderItemsList = orderItemService.list(orderItemsQueryWrapper);
            // 获取商品id列表
            List<Long> goodIds = orderItemsList.stream().map(orderItems -> {
                GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
                return goodSnapshot.getId();
            }).toList();
            QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
            goodQueryWrapper.in("id", goodIds);
            // 获取商品列表
            List<Good> goodList = goodService.list(goodQueryWrapper);
            // 封装为map集合（键：商品id, 值：库存量）,用来存储购买的商品id对应的购买数量
            Map<Long, Integer> map = new HashMap<>();
            for (OrderItems orderItems : orderItemsList) {
                GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
                Long goodId = goodSnapshot.getId();
                Integer quantity = orderItems.getQuantity();
                map.put(goodId, quantity);
            }

            // 批量更新商品库存
            for (Good good : goodList) {
                Long goodId = good.getId();
                // 已经下单购买的数量
                Integer quantity = map.get(goodId);
                // 原有的库存
                Integer inventory = good.getInventory();
                good.setInventory(quantity + inventory);
                good.setUpdateTime(DateUtil.date());
            }
            boolean updateBatch = goodService.updateBatchById(goodList);
            ThrowUtils.throwIf(!updateBatch, ErrorCode.SYSTEM_ERROR, "商品库存恢复失败");

            System.out.println("---------------------------微信退款回调（结束）-------------------------------");
        }


        return true;
    }




    /**
     * 部分退款回调
     */
    @Override
    public boolean refundPartCallback(RefundNotification refundNotification) {
        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderIdByString = refundNotification.getOutTradeNo();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderIdByString);
        Order order = orderService.getOne(queryWrapper);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + orderIdByString);
        }

        // 生成退款记录
        RefundRecord refundRecord = new RefundRecord();
        refundRecord.setOutTradeNo(refundNotification.getOutTradeNo());
        refundRecord.setOutRefundNo(refundNotification.getOutRefundNo());
        refundRecord.setRefundAmount(order.getTotalAmount());

        boolean save = refundRecordService.save(refundRecord);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "退款记录生成失败");


        String outRefundNo = refundNotification.getOutRefundNo();
        String orderItemId = RefundUtils.parseRefundNoToItemId(outRefundNo);

        GoodTypeEnum orderTypeEnum = GoodTypeEnum.getEnumByValue(order.getOrderType());
        if (GoodTypeEnum.SERVICE.equals(orderTypeEnum)) {
            // 更新服务类订单待处理记录
            UpdateWrapper<PendingServiceOrder> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("orderItemId", orderItemId).set("orderItemStatus", OrderStatusConstant.PAYMENT_REFUNDED);
            boolean update = pendingServiceOrderService.update(updateWrapper);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "服务类订单待处理记录状态更新失败");
        } else {
            // 恢复商品库存
            OrderItems orderItems = orderItemService.getById(orderItemId);
            Long id = orderItems.getGoodSnapshot().getId();
            Good good = goodService.getById(id);
            ThrowUtils.throwIf(good == null, ErrorCode.OPERATION_ERROR, "当前商品不存在");
            good.setInventory(good.getInventory() + orderItems.getQuantity());
            boolean update = goodService.updateById(good);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "商品库存恢复失败");
        }
        return true;
    }


//
//    @Override
//    public boolean sendSubscribeMessage(String openid, String templateId) {
//        try {
//            // 获取小程序对象
//            WxMaService wxMaService = wxOpenConfig.getWxMaService();
//            // 获取消息接口
//            WxMaMsgService msgService = wxMaService.getMsgService();
//            // 构建订阅消息体
//            WxMaSubscribeMessage wxMaSubscribeMessage = new WxMaSubscribeMessage();
//            wxMaSubscribeMessage.setToUser(openid);
//            wxMaSubscribeMessage.setTemplateId(templateId);
//            List<WxMaSubscribeMessage.MsgData> msgDataList = getMsgData();
//            wxMaSubscribeMessage.setData(msgDataList);
//            // 发送消息
//            msgService.sendSubscribeMsg(wxMaSubscribeMessage);
//            return true;
//        } catch (Exception e) {
//            log.error("发送订阅消息失败：", e);
//            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "发送订阅消息失败");
//        }
//    }

//    /**
//     * 提取消息数据
//     */
//    private static List<WxMaSubscribeMessage.MsgData> getMsgData() {
//        List<WxMaSubscribeMessage.MsgData> msgDataList = new ArrayList<>();
//        String[][] dataItems = {
//                {"character_string1", "2022060215050983711"},
//                {"time2", "2022-06-22 15:48:05"},
//                {"thing3", "测试店"},
//                {"amount4", "99999元"},
//                {"thing5", "收款到账，请查看"}
//        };
//        for (String[] item : dataItems) {
//            WxMaSubscribeMessage.MsgData msgData = new WxMaSubscribeMessage.MsgData();
//            msgData.setName(item[0]);
//            msgData.setValue(item[1]);
//            msgDataList.add(msgData);
//        }
//        return msgDataList;
//    }

    /**
     * 根据微信官方发送的请求获取信息
     */
    @SneakyThrows
    public NotificationParser getNotificationParser(HttpServletRequest request) {
        System.out.println("---------------------------获取信息-------------------------------");
        // 获取RSA配置
        NotificationParser notificationParser = new NotificationParser(wxPayConfig.getRSAConfig());
        // 构建请求
        StringBuilder bodyBuilder = new StringBuilder();
        BufferedReader reader = request.getReader();
        String line;
        while ((line = reader.readLine()) != null) {
            bodyBuilder.append(line);
        }
        String body = bodyBuilder.toString();
        String timestamp = request.getHeader("Wechatpay-Timestamp");
        String nonce = request.getHeader("Wechatpay-Nonce");
        String signature = request.getHeader("Wechatpay-Signature");
        String singType = request.getHeader("Wechatpay-Signature-Type");
        String wechatPayCertificateSerialNumber = request.getHeader("Wechatpay-Serial");
        requestParam = new RequestParam.Builder()
                .serialNumber(wechatPayCertificateSerialNumber)
                .nonce(nonce)
                .signature(signature)
                .timestamp(timestamp)
                .signType(singType)
                .body(body)
                .build();
        System.out.println(requestParam.toString());
        System.out.println("---------------------------信息获取完毕-------------------------------");
        return notificationParser;
    }
}
